// Copyright (c) 2022 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "dawnopengl.hpp"
#include "game/DawnGame.hpp"
#include "display/RenderManager.hpp"

using namespace Dawn;

RenderManager::RenderManager(DawnGame *game) :
  IRenderManager(game),
  backBuffer(*this),
  renderPipeline(this)
{
}

void RenderManager::init() {
  this->renderPipeline.init();
  
  this->simpleShader.compile();
  this->uiShaderProgram.compile();

  // Prepare the initial values
  glEnable(GL_TEXTURE_2D);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glDepthMask(GL_TRUE);
  glDepthFunc(GL_LESS);
}

RenderTarget * RenderManager::getBackBuffer() {
  return &this->backBuffer;
}

RenderPipeline * RenderManager::getRenderPipeline() {
  return &this->renderPipeline;
}

void RenderManager::setRenderFlags(renderflag_t flags) {
  this->renderFlags = flags;
  
  if((flags & RENDER_MANAGER_RENDER_FLAG_DEPTH_TEST) == 0) {
    glDisable(GL_DEPTH_TEST);
  } else {
    glEnable(GL_DEPTH_TEST);
  }

  if((flags & RENDER_MANAGER_RENDER_FLAG_BLEND) == 0) {
    glDisable(GL_BLEND);
  } else {
    glEnable(GL_BLEND);
  }
}

void RenderManager::update() {
  this->getRenderPipeline()->render();
}

RenderManager::~RenderManager() {
}